-------------------------------------------------------------------------------
-- ElvUI Raid Markers Bar By Lockslap
-- <Borderline Amazing> US, Bleeding Hollow
-------------------------------------------------------------------------------
local E, _, V, P, G, _ = unpack(ElvUI)
local RM = E:NewModule("RaidMarkers")
local L = LibStub("AceLocale-3.0"):GetLocale("ElvUI_RaidMarkers", false)
local M = E:GetModule("Minimap")
local EP = LibStub("LibElvUIPlugin-1.0")

local BUTTON_HEIGHT = 18
local BUTTON_WIDTH = 18
local BUTTON_DISTANCE = 5
local FRAME_HEIGHT = 22
local FRAME_WIDTH = 150

local function InstanceCheck()
	local inInstance, instanceType = IsInInstance()
	if not inInstance or (inInstance and not E.db.general.raidmarkers.instanceType[instanceType]) then
		return false
	else
		return true
	end
end

local function LeaderAssist()
	if UnitIsGroupLeader("player") or UnitIsRaidOfficer("player") then
		return true
	else
		return false
	end
end

local function UpdateBar()
	local cToggle = ElvConfigToggle:GetWidth() or 0
	FRAME_WIDTH = Minimap:GetWidth() + cToggle + 1
	BUTTON_DISTANCE = math.ceil((FRAME_WIDTH - (BUTTON_WIDTH * 9)) / 9)
	_G["ElvRMFrame"]:SetWidth(FRAME_WIDTH)
	_G["ElvRMFrame"]:ClearAllPoints()
	_G["ElvRMFrame"]:SetPoint("TOPLEFT", LeftMiniPanel:IsShown() and LeftMiniPanel or Minimap, "BOTTOMLEFT", LeftMiniPanel:IsShown() and 0 or -1, 1)
	for i = 1, 9 do
		local button = _G[("ElvRMFrameButton%d"):format(i)]
		local prev = _G[("ElvRMFrameButton%d"):format(i - 1)]
		button:ClearAllPoints()
		if i == 1 then
			button:SetPoint("LEFT", 3, 0)
		else
			button:SetPoint("LEFT", prev, "RIGHT", BUTTON_DISTANCE, 0)
		end
	end
end

function RM:UpdateBar()
	-- do the leader/assistant check
	if E.db.general.raidmarkers.assist then
		if (IsInGroup() or IsInRaid()) and not LeaderAssist() then
			if ElvRMFrame:IsShown() then
				ElvRMFrame:Hide()
			end
			return
		end
	end
	
	if E.db.general.raidmarkers.mode == "instance" then
		if not InstanceCheck() then
			if ElvRMFrame:IsShown() then
				ElvRMFrame:Hide()
			end
		elseif not ElvRMFrame:IsShown() then
			ElvRMFrame:Show()
		end
	elseif E.db.general.raidmarkers.mode == "target" then
		if E.db.general.raidmarkers.instance then
			if not InstanceCheck() then
				if ElvRMFrame:IsShown() then
					ElvRMFrame:Hide()
				end
			else
				if UnitExists("target") and not ElvRMFrame:IsShown() then
					ElvRMFrame:Show()
				elseif not UnitExists("target") and ElvRMFrame:IsShown() then
					ElvRMFrame:Hide()
				end
			end
		else
			if UnitExists("target") and not ElvRMFrame:IsShown() then
				ElvRMFrame:Show()
			elseif not UnitExists("target") and ElvRMFrame:IsShown() then
				ElvRMFrame:Hide()
			end
		end
	end
end

function RM:Initialize()
	-- figure out the distance between the icons
	local cToggle = ElvConfigToggle:GetWidth() or 0
	FRAME_WIDTH = Minimap:GetWidth() + cToggle + 1
	BUTTON_DISTANCE = math.ceil((FRAME_WIDTH - (BUTTON_WIDTH * 9)) / 9)
	
	local f = CreateFrame("Frame", "ElvRMFrame", E.UIParent)
	f:SetResizable(false)
	f:SetHeight(FRAME_HEIGHT)
	f:SetWidth(FRAME_WIDTH)
	f:SetTemplate("Default", true)
	f:SetPoint("TOPLEFT", LeftMiniPanel:IsShown() and LeftMiniPanel or Minimap, "BOTTOMLEFT", LeftMiniPanel:IsShown() and 0 or -1, 1)
	
	-- create the buttons
	for i = 1, 8 do
		local button = CreateFrame("Button", ("ElvRMFrameButton%d"):format(i), f)
		button:EnableMouse(true)
		button:SetHeight(BUTTON_HEIGHT)
		button:SetWidth(BUTTON_WIDTH)
		button:SetScript("OnClick", function(self, button)
			if GetRaidTargetIndex("target") ~= i then
				SetRaidTarget("target", i)
			else
				SetRaidTarget("target", 0)
			end
		end)
		
		if i == 1 then
			button:SetPoint("LEFT", 3, 0)
		else
			local prev = _G[("ElvRMFrameButton%d"):format(i - 1)]
			button:SetPoint("LEFT", prev, "RIGHT", BUTTON_DISTANCE, 0)
		end
		
		local image = button:CreateTexture(nil, "BACKGROUND")
		image:SetAllPoints()
		image:SetTexture(("Interface\\TargetingFrame\\UI-RaidTargetingIcon_%d"):format(i))
		
		local highlight = button:CreateTexture(nil, "HIGHLIGHT")
		highlight:SetAllPoints(image)
		highlight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")
		highlight:SetTexCoord(0, 1, 0.23, 0.77)
		highlight:SetBlendMode("ADD")
	end
	
	-- remove button
	local cancel = CreateFrame("Button", "ElvRMFrameButton9", f)
	cancel:EnableMouse(true)
	cancel:SetHeight(BUTTON_HEIGHT)
	cancel:SetWidth(BUTTON_WIDTH)
	cancel:SetPoint("LEFT", _G["ElvRMFrameButton8"], "RIGHT", BUTTON_DISTANCE, 0)
	cancel:SetScript("OnClick", function(self, button) SetRaidTarget("target", 0) end)
	
	local cImage = cancel:CreateTexture(nil, "BACKGROUND")
	cImage:SetAllPoints()
	cImage:SetTexture("Interface\\BUTTONS\\UI-GroupLoot-Pass-Up")
	
	local cHighlight = cancel:CreateTexture(nil, "HIGHLIGHT")
	cHighlight:SetAllPoints(cImage)
	cHighlight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")
	cHighlight:SetTexCoord(0, 1, 0.23, 0.77)
	cHighlight:SetBlendMode("ADD")
	
	hooksecurefunc(M, "UpdateSettings", UpdateBar)
	self:UpdateBar()
end

E:RegisterModule(RM:GetName())

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_TARGET_CHANGED")
frame:SetScript("OnEvent", function(self, event, ...)
	-- do the leader/assistant check
	if E.db.general.raidmarkers.assist then
		if (IsInGroup() or IsInRaid()) and not LeaderAssist() then
			if ElvRMFrame:IsShown() then
				ElvRMFrame:Hide()
			end
			return
		end
	end
	
	if E.db.general.raidmarkers.mode == "target" then
		local inInstance, instanceType = IsInInstance()
		if E.db.general.raidmarkers.instance then
			if not InstanceCheck() then
				if ElvRMFrame:IsShown() then
					ElvRMFrame:Hide()
				end
				return
			end
		end
		if UnitExists("target") and not ElvRMFrame:IsShown() then
			ElvRMFrame:Show()
		elseif not UnitExists("target") and ElvRMFrame:IsShown() then
			ElvRMFrame:Hide()
		end
	end
end)

P["general"]["raidmarkers"] = {
	["mode"] = "target",
	["instance"] = false,
	["instanceType"] = {
		["arena"] = true,
		["party"] = true,
		["pvp"] = true,
		["raid"] = true,
		["assist"] = false,
	},
}

local function InjectOptions()
	E.Options.args.general.args.raidmarkers = {
		order = 6,
		type = "group",
		name = L["Raid Markers Bar"],
		guiInline = true,
		get = function(info) return E.db.general.raidmarkers[info[#info]] end,
		set = function(info, value) E.db.general.raidmarkers[info[#info]] = value; RM:UpdateBar() end,
		args = {
			mode = {
				order = 1,
				type = "select",
				name = L["Display Mode"],
				desc = L["Set how you want the bar displayed."],
				values = {
					["instance"] = L["In Instance Only"],
					["target"] = L["Target Only"],
				}
			},
			instance = {
				order = 2,
				type = "toggle",
				name = L["Instance Only"],
				desc = L["Only show the raid markers bar when you have a target in an instance.\n\n|cff00ff96This only applies if you have the display mode set to Target Only.|r"],
				disabled = function() return E.db.general.raidmarkers.mode ~= "target" end,
			},
			assist = {
				order = 3,
				type = "toggle",
				name = L["Leader/Assist Required"],
				desc = L["Only show the raid markers bar if you are the raid leader, or a raid assistant.\n\n|cff00ff96This applies to both modes.|r"],
			},
			instanceType = {
				order = -1,
				type = "multiselect",
				name = L["Instance Types"],
				desc = L["Types of instances to show the bar in.\n\n|cff00ff96This applies to both modes.|r"],
				values = {
					["arena"] = L["Arenas"],
					["pvp"] = L["Battlegrounds"],
					["party"] = L["Dungeons"],
					["raid"] = L["Raids"],
				},
				get = function(info, key) return E.db.general.raidmarkers[info[#info]][key] end,
				set = function(info, key, value) E.db.general.raidmarkers[info[#info]][key] = value; RM:UpdateBar() end,
			},
		},
	}
end

EP:RegisterPlugin(..., InjectOptions)
	
SlashCmdList["ELVRAIDMARKERS"] = function(msg, editbox)
	if not ElvRMFrame:IsShown() then
		ElvRMFrame:Show()
	else
		ElvRMFrame:Hide()
	end
end
SLASH_ELVRAIDMARKERS1 = "/erm"